﻿using Syncfusion.Data;
using Syncfusion.WinForms.DataGrid;
using Syncfusion.WinForms.Input;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Data.SqlClient;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using DemoCommon.Grid;
using System.Windows.Forms.VisualStyles;
using Syncfusion.WinForms.DataGrid.Events;
using Syncfusion.WinForms.Input.Enums;
using Syncfusion.WinForms.DataGrid.Enums;
using System.Globalization;
using Syncfusion.WinForms.ListView;
using Syncfusion.WinForms.ListView.Enums;
using Syncfusion.WinForms.DataGrid.Renderers;
using System.Reflection;

namespace ColumnTypes
{
    public partial class Form1 : Form
    {
        #region Fields
        OrderInfoCollection orderInfo;
        #endregion

        #region Constuctor
        public Form1()
        {
            InitializeComponent();
            orderInfo = new OrderInfoCollection();
            NumberFormatInfo nfi = new NumberFormatInfo();
            nfi.NumberDecimalDigits = 0;
            nfi.NumberGroupSizes = new int[] { };
            this.sfDataGrid1.DataSource = orderInfo.OrdersListDetails;
            this.sfDataGrid1.Columns.Add(new GridNumericColumn() { MappingName = "OrderID", HeaderText = "Order ID", NumberFormatInfo = nfi });
            this.sfDataGrid1.Columns.Add(new GridTextColumn() { MappingName = "CustomerID", HeaderText = "Customer ID" });
            this.sfDataGrid1.Columns.Add(new GridTextColumn() { MappingName = "ContactNumber", HeaderText = "Contact Number" });
            this.sfDataGrid1.Columns.Add(new GridCheckBoxColumn() { MappingName = "IsClosed", HeaderText = "Closed", AllowCheckBoxOnHeader = true, CheckBoxSize = new Size(13, 13), Width = 100 });
            this.sfDataGrid1.Columns["ContactNumber"].CellStyle.HorizontalAlignment = HorizontalAlignment.Right;
            NumberFormatInfo numberFormat = Application.CurrentCulture.NumberFormat.Clone() as NumberFormatInfo;
            numberFormat.CurrencyDecimalDigits = 0;
            numberFormat.CurrencyGroupSizes = new int[] { };

            this.sfDataGrid1.Columns.Add(new GridNumericColumn()
            {
                MappingName = "UnitPrice",
                HeaderText = "Unit Price",
                FormatMode = FormatMode.Currency,
                NumberFormatInfo = numberFormat
            });

            numberFormat = Application.CurrentCulture.NumberFormat.Clone() as NumberFormatInfo;
            numberFormat.NumberDecimalDigits = 0;
            numberFormat.NumberGroupSizes = new int[] { };
            this.sfDataGrid1.Columns.Add(new GridNumericColumn()
            {
                HeaderText = "Quantity",
                MappingName = "Quantity",
                FormatMode = FormatMode.Numeric,
                NumberFormatInfo = numberFormat
            });

            numberFormat = Application.CurrentCulture.NumberFormat.Clone() as NumberFormatInfo;
            numberFormat.PercentDecimalDigits = 0;
            numberFormat.NumberGroupSizes = new int[] { };
            this.sfDataGrid1.Columns.Add(new GridNumericColumn() { MappingName = "Discount", HeaderText = "Discount", FormatMode = FormatMode.Percent, NumberFormatInfo = numberFormat });
            this.sfDataGrid1.Columns.Add(new GridDateTimeColumn()
            {
                MappingName = "OrderDate",
                HeaderText = "Order Date",
                FilterMode = ColumnFilter.DisplayText,
                DateTimeEditingMode = DateTimeEditingMode.Mask,
                Pattern = DateTimePattern.Custom,
                Format = "dd-MM-yy"
            });

            this.sfDataGrid1.Columns.Add(new GridHyperlinkColumn() { MappingName = "Hyperlink", HeaderText = "Order URI" });
            
            this.sfDataGrid1.Columns.Add(new GridComboBoxColumn() { MappingName = "ShipCityID", HeaderText = "Ship City", DisplayMember = "ShipCityName", ValueMember = "ShipCityID", DropDownStyle = DropDownStyle.DropDownList, DataSource = orderInfo.ShipCityDetails });
            this.sfDataGrid1.CellRenderers.Remove("DateTime");
            this.sfDataGrid1.CellRenderers.Add("DateTime", new CustomDateTimeCellRenderer());
            this.sfDataGrid1.SelectionMode = GridSelectionMode.Extended;
            this.sfDataGrid1.CellCheckBoxClick += sfDataGrid1_CellCheckBoxClick;
        }

        void sfDataGrid1_CellCheckBoxClick(object sender, CellCheckBoxClickEventArgs e)
        {
            var selectedRows = this.sfDataGrid1.View.Records.Where(item => (item.Data as OrderInfo).IsClosed == true).ToList();
            var checkedItems = selectedRows.Count + (e.NewValue == CheckState.Checked ? 1 : -1);

            GridCheckBoxColumn column = e.Column as GridCheckBoxColumn;
            PropertyInfo info = column.GetType().GetProperty("HeaderState", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public);

            if (checkedItems == this.sfDataGrid1.View.Records.Count)
            {
                info.SetValue(column, CheckState.Checked);
            }
            else
            {
                info.SetValue(column, CheckState.Unchecked);
            }            
        }

        #endregion

        private void button1_Click(object sender, EventArgs e)
        {
            if (this.sfDataGrid1.SelectedItems.Count > 0)
            {
                foreach (OrderInfo selectedItem in this.sfDataGrid1.SelectedItems)
                {
                    if (selectedItem.IsClosed == true)
                        selectedItem.IsClosed = false;
                }
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            var selectedRows = this.sfDataGrid1.View.Records.Where(item => (item.Data as OrderInfo).IsClosed == true).ToList();
            foreach (var row in selectedRows)
            {

            }

            MessageBox.Show("selected checkbox rows count : " + selectedRows.Count);
        }
    }


    public class CustomDateTimeCellRenderer : GridDateTimeCellRenderer
    {
        protected override void OnInitializeEditElement(DataColumnBase column, Syncfusion.WinForms.GridCommon.ScrollAxis.RowColumnIndex rowColumnIndex, SfDateTimeEdit uiElement)
        {
            base.OnInitializeEditElement(column, rowColumnIndex, uiElement);
            SfDateTimeEdit dateTimeEdit = uiElement as SfDateTimeEdit;
            dateTimeEdit.Style.BackColor = Color.AliceBlue;
            dateTimeEdit.Style.ForeColor = Color.Red;
            dateTimeEdit.MonthCalendar.Style.Cell.CellBackColor = Color.Wheat;
        }
    }
}
